
class AI_player {
  int computerID;
  
// Konstruktor //
  AI_player(int id){
    computerID = id;
  }



/***********************
  Funktionsbeschreibung: 
  Diese Funktion bekommt das aktuelle spielfeld übergeben und wählt den best möglichen Zug aus          
***********************/
int[] chooseTurn (Feld[][] board){
 
 int[] bestZug = new int[3];     // bestZug[0] = bester Score; bestZug[1] = Spalte; bestZug[2] = Zeile                                                                              
 bestZug[0] = -20;               // -20 sei der Default-Wert für den Score.

 int turnsLeft = 0;              // varibable die speichert wie viele Züge verbleiben
 int player = computerID;        // aktueller spieler wird bestimmt 
 
 // Anzahl der verbleibenden züge wird ermittelt
  for (int s=0; s<3; s++)
    for (int z=0; z<3; z++)
      if (board[s][z].getStatus() == 0)                      
        turnsLeft++;

    for (int s = 0; s < board.length; s++){      
      for (int z = 0; z < board[s].length; z++){ 
        if (board[s][z].getStatus() == 0){     // WENN ein Feld unbelegt                     
          board[s][z].setStatus(player);       // belege das Feld mit dem spieler (dies ist nur temporär)       
          int alpha = -20;                     // -20 startwert für alpha (maximum)
          int beta = 20;                       // 20  startwert für beta  (minimum)
          int score = alphabeta(board, getGegner(player), turnsLeft , alpha, beta);  // berechne den Score des Feldes mit hilfe des AlphaBeta-Algorithmus (siehe alphabeta() )
          board[s][z].setStatus(0);            // Mache das Feld wieder frei       
          if (score > bestZug[0]){             // prüfe ob aktueller Score > bester Score, ist die der Fall dann speichere das Feld als besten Zug
            bestZug[0] = score;
            bestZug[1] = s;
            bestZug[2] = z;
          }
        }
    } }
  
  int[] rueckgabe = {bestZug[1], bestZug[2]}; // die Spalte und die Zeile des besten Zuges werden zurück gegeben
  return rueckgabe;
}

/***********************
  Parameter:
    board  -> Spielbrett
    player -> Spieler der am Zug ist
    turnsLeft -> Verbleibende Züge
    alpha  -> maximalwert der höheren ordnung
    beta   -> minimalwert der höheren ordnung
  
  Funktionsbeschreibung: 
  
  Als ersten Schritt berechnet Der algorithmus einen Punktewert für jedes mögliche ergebnis des Spiels (also entweder unentschieden oder)

***********************/
int alphabeta(Feld[][] board, int player, int turnsLeft, int alpha, int beta){
  int winner = spielfeld.gewonnen(board); // es wird geprüft ob es auf dem übergebenen Spielfeld einen gewinner gibt, dies ist der fall wenn die funktion gewonnen() ungleich 0 ist
  int punkteAbzug = 9 - turnsLeft;        // Der Computer soll schnelle(also in wenigen zügen) Siege/Niederlagen schwerer gewichten. Ein schneller Sieg soll also mehr Pluspunkte und eine schnelle Niederlage mehr Minuspunkte zurück geben
  
  // Rekursionsabbruch: Ist der algrithmus bei einem spielfeld angekommen bei dem das spiel vorbei ist wird ein Score für das Spielergebnis bestimmt, dabei gilt: 
  // bei unentschieden -> score = 0
  // bei sieg für den Computer -> score echt positiv und je früher im spiel desdo höher
  // bei Niederlage für den Computer -> score echt negativ und je früher im spiel desdo niedriger
  if (gameOver(board) || winner != 0){
    int score = 0;
    if (winner == getGegner(computerID))  score = -10 + punkteAbzug;  // Computer Verloren
    if (winner == 0)             score = 0;                           // Unentschieden 
    if (winner == computerID)        score = 10 - punkteAbzug;        // Computer Gewonnen
      
    return score; // da dies der Rekursionsabbruch ist muss an dieser stelle auch eine rückgabe sein.
  } 
  // WENN das Spiel noch nicht vorbei ist dann wird auf jedem feld ein zug getätigt und die alphabeta funktion rekursiv aufgerufen, so das ein Score zurückgegeben wird. 
  // Ist nun der Computer am zug, sucht der Algorithmus den maximalen Score der unterschiedlichen Felder und gibt diesen als Rückgabe zurück, außerdem wird geschaut ob der Minimalwert des aufrufenden Algorythmus kleiner ist als der aktuelle Maximalwert, ist dies der Fall wird nicht weiter nach einem Maximum gesucht. Grund dafür ist das aufrufende Algorithmus nach dem kleinsten Score sucht und den aktuellen Wert deshalb garnicht beachten wird.  
  // Ist der Spieler am zug, sucht der Algorithmus den minimalen Score der unterschiedlichen Felder und gibt diesen als Rückgabe zurück, außerdem wird geschaut ob der Maximalwert des aufrufenden Algorythmus größer ist als der aktuelle Minimalwert, ist dies der Fall wird nicht weiter nach einem Minimum gesucht. Grund dafür ist das aufrufende Algorithmus nach dem größten Score sucht und den aktuellen Wert deshalb garnicht beachten wird.  
  else {        
    for (int s=0; s < board.length; s++) {    // Für jedes Feld...
      for (int z=0; z < board.length; z++) { 
        if (board[s][z].getStatus() == 0){      // Wenn Feld leer
          board[s][z].setStatus(player);         // Feld wird mit dem aktuellen spieler belegt
          int score = alphabeta(board, getGegner(player), turnsLeft-1, alpha, beta);  // alphabeta wird rekursiv aufgerufen, dabei ist zu beachten das in der vorherigen zeile ein Spielzug ausgeführt wurde und sich das board dementsprechened verändert hat, deshalb wird die funktion auch mit dem gegenSpieler aufgerufen der ja nun am zug ist
          board[s][z].setStatus(0);          // Feld wird wieder freigemacht
           
          if (player == computerID){  // Wenn Computer am Zug, maximiere den Score
            if (score > alpha)        // score > maximum -> score neues maximum
              alpha = score;
            if (alpha >= beta)        // Abbruch des Algrithmus. Die aufrufende/höhere Instanz sucht das Minimum und hat schon einen Wert (beta) der kleiner ist als der aktuelle Score je werden kann.
              return beta;
          } else {                    // Wenn Spieler am Zug, minimiere den Score (gleiche funktionsweise wie Score maximieren)
            if (score < beta)         
              beta = score;
           if (beta <= alpha)        // Abbruch des Algrithmus. Die aufrufende/höhere Instanz sucht dasMaximum und hat schon einen Wert (Alpha) der größer ist als der aktuelle Score je werden kann. 
              return alpha;
          }   
        }         
    } }
  }
   if (player == computerID)  // Rückgabe: wenn computer am zug dann wird der größt mögliche Score zurück gegeben, sonst der kleinst mögliche (Dies ist der fall wenn der Spieler am zug ist)
     return alpha;
   else 
     return beta;
}

/***********************
  Funktionsbeschreibung: gibt den Gegner des übergebenen Spielers zurück
***********************/
int getGegner(int player){
  if (player == 1)
    return 2;
  else 
    return 1;
}

/***********************
  Funktionsbeschreibung: Prüft ob das Spiel vorbei ist (false wenn nicht)
***********************/
boolean gameOver(Feld[][] board){
  boolean result = true;
  
  for (int s=0; s < board.length; s++)
    for (int z=0; z < board[s].length; z++)
      if (board[s][z].getStatus() == 0)
        result = false;
  return result; 
}

}