// Packages die für die Kommunikation mit dem roboter über wlan benötigt werden
import java.io.*;  
import java.net.*;  

// Identifikationen der Spieler werden festgelegt
int player_ID = 1;   
int computer_ID = 2;
// größe des Spielfeldes auf dem bildschirm (NICHT die größe des Fensters)
int gameSize = 600;
// Ein Objekt der Klasse Spiel wird deklariert, initialisierung geschieht in "setup()"
Spiel spielfeld;
// Ein Objekt der Klasse AI_player wird erzeugt, über dieses wird später entschieden welchen Zug der computer wählt
AI_player computer = new AI_player(computer_ID);

/***********************
  Funktionsbeschreibung:
  Wenn eine Taste gedrückt wird und das Match nicht am laufen ist, dann starte, jenachdem welche taste gedrückt wurde, ein Match mit dem entsprechenden Startspieler
***********************/
void keyPressed(){
  if (spielfeld.getStatus() == 0){   
    if (key == '1')                          // Taste 1: Der Spieler beginnt das Match
      spielfeld = new Spiel(gameSize, player_ID);
    else if (key == '2')                     // Taste 2: Der Computer beginnt das Match
      spielfeld = new Spiel(gameSize, computer_ID);
    else {                                   // andere Taste: Der erste Spieler wird zufällig gewählt
      int zufall = int(random(player_ID, computer_ID+1)); // wählt Zufallszahl aus dem Intervall [1,3[ und wandelt diesen in ein Integerwert um ( nur 1 oder 2 möglich)
      spielfeld = new Spiel(gameSize, zufall);
    }
  }
}

/***********************
  Funktionsbeschreibung:  
  Wurde ein Match gestartet wird in dieser Funktion der größte Teil des Programms durchgeführt. Folgendes wird gemacht:
  1.1. ist der Spieler am Zug dann warte auf eine gültige Mauseingabe
  1.2. ist der Computer am zug dann fordere diesen auf, sich für einen Zug zu entscheiden
  2. Es wird gespeichert ob und welcher zug gemacht wurde
  3. Das Spielfeld wird auf dem bildschirm angezeigt
  4. Wurde ein Zug gewählt wird der zug ausgeführt, an den Roboter übermittelt, der Spieler gewechselt und die Zugzahl erhöht
  5. Es wird geprüft ob das Spiel vorbei ist, wenn das der Fall ist dann wird der Gewinner ausgegeben, die Statistik aktualisiert und evtl eine neue Runde gestartet 
***********************/
void draw(){
  boolean moveMade = false;  // Merker in dem gespeichert wird ob ein zug gemacht wurde 
  int s = -1;  // Variablen in denen der gemachte Zug gespeichert wird.
  int z = -1;
  
  
  if (spielfeld.getStatus() == 1){  // WENN ein Match gestartet wurde
   
    // 1.1. // WENN der Spieler am Zug ist 
    if(spielfeld.getPlayer() == player_ID){  
      if(mousePressed){  // WENN maus gedrückt wurde
         int[] daten = mausKlick(mouseX, mouseY); // Mauskoordinaten werden an mausKlick übergeben, diese funktion gibt dann zurück welches feld angeklickt wurde und ob die wahl gültig war
         
         // 2. //
         moveMade = (daten[0] != -1); //
         s = daten[0];
         z = daten[1];
      }
    }
    // 1.2. // WENN Computer am Zug ist 
    if(spielfeld.getPlayer() == computer_ID){
     int[] zugwahl= computer.chooseTurn(spielfeld.getBoard()); // Computer wählt seinen Zug und übergibt diesen
    
     // 2. //
     moveMade = true;
     s = zugwahl[0];
     z = zugwahl[1];
    }  
    
    // 3. //
    zeigeSpiel(spielfeld.getBoard());
    
    // 4. //
    if (moveMade){
      spielfeld.makeMove(s, z);               // gewählter zug wird durchgeführt
      posToRobot(s,z, spielfeld.getPlayer()); // zug wird an Roboter übermittelt
      while (robotFertigTMP() == false){  // es wird gewartet bis der Roboter fertig mit dem Zeichnen ist.
        delay(100);
        print("- ");
      }
      spielfeld.switchPlayer(); // aktiver Spieler wird gewechselt
      spielfeld.incTurn();      // Zugzahl wird erhöht 
    }
    
    // 5. //
    int winner = spielfeld.gewonnen(spielfeld.getBoard()); // gewinner wird ermittelt und gespeichert
    if (winner != 0 || (winner == 0 && spielfeld.getTurn() >= 9)){  // WENN einer der Spieler gewonnen hat (winner ungleich 0) oder gibt es keine möglichen züge mehr...
      printWinner(winner); // gebe Gewinner auf dem bildschrim aus
      fancyText(gameSize/2, "Zum neu starten beliebige Taste drücken",gameSize/24, #00AA00); // frage nutzer nach neuer runde
      if (keyPressed){ 
        freshStats(winner); // aktualisiere Spielstand
        setup();  // resette das Spiel
      }  
    }
  }
  showStats(); // zeige Spielstand an
  }
  
/***********************
  Funktionsbeschreibung:
  Funktion wird dazu benötigt um das spiel zu starten oder das spiel zu resetten
***********************/
void setup(){
  
  // Spielfeld wird initialisiert aber noch nicht gestartet
  spielfeld = new Spiel();
  
  
  size(900,600); // Fenstergröße wird festgelegt
  clearSpace();  // Alte Bildschirmausgaben werden überschrieben
  setupRoCom();  // Kommunikation zum Roboter wird vorbereitet (siehe klasse RoCom)
  
  // Willkommensbildschirm
  fancyText(gameSize/3, "Drücke 1 oder 2 um Spiel zu starten", gameSize/20, #000000);   
  fancyText(gameSize/2-gameSize/20, "1 - Du beginnst", gameSize/30, #FF0000);          
  fancyText(gameSize/2, "2 - Der Computer beginnt", gameSize/30, #0000FF); 
  fancyText(gameSize/2 + gameSize/20, "Andere Taste - Zufällig", gameSize/30, #00AA00); 
}

/***********************
  Funktionsbeschreibung:
  Temporäre funktion die zum testen des programmes ohne Roboter gebraucht wird
***********************/
boolean robotFertigTMP (){
  // es werden mehrere durchgänge simuliert
int i = int(random(3));
if (i == 1)
 return true;
 else 
 return false;
}

/***********************
  Funktionsbeschreibung:
  
***********************/